import { FormMDCtrlController } from '@ibiz-template/controller';
import { FormMDCtrlModel } from '@ibiz-template/model';
import { useController, useNamespace } from '@ibiz-template/vue-util';
import { defineComponent, getCurrentInstance, h, ref } from 'vue';
import './form-mdctrl-form-style2.scss';

export const FormMDCtrlFormStyle2 = defineComponent({
  name: 'FormMDCtrlFormStyle2',
  props: {
    controller: {
      type: FormMDCtrlController,
      required: true,
    },
    modelData: {
      type: FormMDCtrlModel,
      required: true,
    },
  },
  setup(props) {
    const ns = useNamespace('form-mdctrl-form-style2');
    const c = props.controller;
    // 激活项
    const active = ref(0);
    const { proxy } = getCurrentInstance()!;

    useController(proxy, props.controller);

    // 点击删除
    const handleRemove = async (
      item: IData,
      index: number,
      event: MouseEvent,
    ) => {
      event.stopPropagation();
      event.preventDefault();
      const length = c.contentCtrlData.length;
      await c.handleRemove(item, index);
      if (
        length !== c.contentCtrlData.length &&
        index <= active.value &&
        active.value !== 0
      ) {
        active.value -= 1;
      }
    };

    // 切换分页
    const activeChange = (index: number) => {
      active.value = index;
      c.force();
    };

    // 绘制头部
    const rendertanHeader = () => {
      const tabtitle = props.modelData.source.userTag;
      return (
        <div class={ns.e('header')}>
          {c.contentCtrlData.map((item: IData, index: number) => {
            const label =
              item.srfmajortext || (tabtitle && item[tabtitle])
                ? item[tabtitle]
                : '新建';
            return (
              <div
                class={[
                  ns.e('header-item'),
                  ns.is('actived', index === active.value),
                ]}
                onClick={() => activeChange(index)}
              >
                <div class={ns.em('header-item', 'title')}>{label}</div>
                <div
                  class={ns.em('header-item', 'delete')}
                  onClick={(event: MouseEvent) =>
                    handleRemove(item, index, event)
                  }
                >
                  <ion-icon name='close-outline' title='关闭'></ion-icon>
                </div>
              </div>
            );
          })}
          <div
            class={[ns.e('header-item'), ns.e('add')]}
            onClick={() => c.handleAdd()}
          >
            <span>添加</span>
            <ion-icon name='add-outline'></ion-icon>
          </div>
        </div>
      );
    };

    // 绘制内容
    const renderTabs = () => {
      return c.contentCtrlData.map((item: IData, index: number) => {
        let formComponent = null;
        const { form } = c.model;
        if (form && c.providers[form.name] && c.model.appEntity) {
          const tempContext = Object.assign(c.form.context.clone(), {
            [c.model.appEntity.codeName?.toLowerCase()]:
              item[c.model.appEntity.keyName],
          });
          formComponent = h(c.providers[form.name].component, {
            props: {
              modelData: form,
              context: tempContext,
              params: c.form.params,
              enableAutoSave: true,
              silentVerify: true,
            },
            on: {
              neuronInit: c.onNeuronInit(item.srfkey),
            },
          });
        }
        return (
          <div
            class={[ns.e('content'), ns.is('actived', index === active.value)]}
            key={item.srfkey}
          >
            {formComponent}
          </div>
        );
      });
    };
    return { ns, rendertanHeader, renderTabs };
  },
  render() {
    return (
      <div class={this.ns.b()}>
        {this.rendertanHeader()}
        {this.renderTabs()}
      </div>
    );
  },
});
